use std::cell::RefCell;

use kahlo::{math::PixelBox, prelude::*};

use crate::{
    component::Component,
    layout::{LayoutNode, LayoutNodeAccess, LeafLayoutNode, SizePolicy},
    theme::Theme,
    ui::UIHandle,
    window::RenderFormat,
};

use super::Widget;

pub struct Label<C: Component> {
    lnode: LeafLayoutNode,
    text: Option<String>,
    rendered: RefCell<Option<kahlo::Alphamap>>,
    _ghost: std::marker::PhantomData<C>,
}

impl<C: Component> Label<C> {
    fn construct(mut node: LayoutNode, text: Option<&str>) -> Self {
        node.set_width_policy(SizePolicy::fixed(0))
            .set_height_policy(SizePolicy::fixed(0));
        Self {
            lnode: LeafLayoutNode::new(node),
            text: text.map(ToString::to_string),
            rendered: None.into(),
            _ghost: Default::default(),
        }
    }

    pub fn new(uih: &UIHandle) -> Self {
        Self::construct(uih.new_layout_node(), None)
    }

    pub fn new_with_text(uih: &UIHandle, text: &str) -> Self {
        Self::construct(uih.new_layout_node(), Some(text))
    }

    pub fn new_with_node_and_text(node: LayoutNode, text: &str) -> Self {
        Self::construct(node, Some(text))
    }

    pub fn set_text(&mut self, text: &str) {
        if Some(text) != self.text.as_deref() {
            self.text = Some(text.to_string());
            self.rendered.take();
        }
    }
}

impl<C: Component> Label<C> {
    fn update_hints(&mut self, uih: &UIHandle) {
        let Some(text) = self.text.as_ref() else {
            return;
        };
        let line = uih.theme().make_line(text.as_str());
        let (rendered, offset) = line.render_line();
        let sz = rendered.size();
        *self.rendered.borrow_mut() = Some(rendered);
        self.lnode.render_needed();

        let wp = self.lnode.width_policy().with_minimum(sz.width as usize);
        self.lnode.set_width_policy(wp);
        let hp = self.lnode.height_policy().with_minimum(sz.height as usize);
        self.lnode.set_height_policy(hp);
    }
}

impl<C: Component> Widget<C> for Label<C> {
    fn poll(
        &mut self,
        uih: &mut UIHandle,
        _input_state: Option<&crate::input::InputState>,
    ) -> Vec<<C as Component>::Msg> {
        if self.rendered.borrow().is_none() {
            self.update_hints(uih);
        }

        vec![]
    }

    fn layout_node(&self) -> LayoutNodeAccess {
        LayoutNodeAccess::new(&self.lnode)
    }
    fn layout_node_mut(&mut self) -> &mut LayoutNode {
        &mut self.lnode
    }

    fn render(&self, theme: &Theme, surface: &mut kahlo::BitmapMut<RenderFormat>) {
        if !self.lnode.render_check() {
            return;
        }
        let rcon = self.rendered.borrow();
        let Some(rendered) = rcon.as_ref() else {
            return;
        };
        let render_location = self.layout_node().render_area().unwrap();
        let render_location = render_location.inner_box(self.lnode.margin());

        surface.fill_region_masked(
            &rendered,
            PixelBox::from_size(rendered.size()),
            render_location.min,
            theme.foreground,
            kahlo::colour::BlendMode::SourceAlpha,
        );
    }
}
