use std::cell::RefCell;

use crate::{prelude::*, layout::{LeafLayoutNode, LayoutNodeAccess, LayoutNode, SizePolicy}, ui::UIHandle, input::{MouseButton, MouseCheckStatus}};
use kahlo::{prelude::*, math::PixelBox};
use winit::keyboard::{Key, NamedKey};

pub struct TextEdit<C: Component> {
    lnode: LeafLayoutNode,
    text: String,
    text_shift: usize,
    cursor_pos: Option<usize>,
    rendered: RefCell<Option<(kahlo::Alphamap, i32)>>,
    onchange: Option<Box<dyn Fn(&str) -> Option<C::Msg>>>,
}

impl<C: Component> TextEdit<C> {
    pub fn new(uih: &UIHandle) -> Self {
        let minheight = (uih.theme().ui_font_size * 1.5).round() as usize;
        let mut lnode = uih.new_layout_node();
        lnode
            .set_width_policy(SizePolicy::expanding(1))
            .set_height_policy(SizePolicy::fixed(minheight));

        Self {
            lnode: LeafLayoutNode::new(lnode),
            text: String::new(),
            text_shift: 0,
            cursor_pos: None,
            rendered: None.into(),
            onchange: None
        }
    }

    pub fn set_text(&mut self, to: &str) {
        self.text = to.to_string();
    }

    fn update_hints(&mut self, uih: &UIHandle) {
        let line = uih.theme().make_line(self.text.as_str());
        let (rendered, offset) = line.render_line();
        let sz = rendered.size();
        *self.rendered.borrow_mut() = Some((rendered, offset));
        self.lnode.render_needed();

        let wp = self.lnode.width_policy().with_minimum(sz.width as usize);
        self.lnode.set_width_policy(wp);
        let hp = self.lnode.height_policy().with_minimum(sz.height as usize);
        self.lnode.set_height_policy(hp);
    }
}

impl<C: Component> Widget<C> for TextEdit<C> {
    fn poll(&mut self, uih: &mut UIHandle, input_state: Option<&crate::input::InputState>) -> Vec<<C as Component>::Msg> {
        let Some(istate) = input_state else { return vec![] };

        if istate.is_focused(self.lnode.id()) {
            if let Some(kp) = &istate.keypress {
                match kp {
                    Key::Named(NamedKey::Backspace) => {
                        self.text.pop();
                        self.lnode.render_needed();
                        self.update_hints(uih);
                    },
                    Key::Named(NamedKey::Delete) => {
                        self.text.pop();
                    },
                    Key::Named(NamedKey::Enter) => {
                        istate.clear_focus();
                    },
                    _ => {
                        if let Some(inp) = &istate.text_input {
                            println!("appending text input {inp:x?}");
                            self.text.push_str(inp.as_str());
                            self.lnode.render_needed();
                            self.update_hints(uih);
                        }
                    }
                }
            }
        }
        if istate.mouse_status_in(self.lnode.render_area().unwrap(), MouseButton::Left) == MouseCheckStatus::Click {
            istate.focus_on(self.lnode.id());
        }

        self.update_hints(uih);

        vec![]
    }

    fn layout_node(&self) -> LayoutNodeAccess {
        LayoutNodeAccess::new(&self.lnode)
    }
    fn layout_node_mut(&mut self) -> &mut LayoutNode {
        &mut self.lnode
    }

    fn render(&self, theme: &crate::theme::Theme, target: &mut kahlo::BitmapMut<crate::window::RenderFormat>) {
        if self.lnode.render_check() {
            let area = self.lnode.render_area().unwrap();
            target.fill_region(area, theme.background);

            let amap = self.rendered.borrow();
            let Some((amap, offset)) = amap.as_ref() else {
                return;
            };

            target.fill_region_masked(amap, PixelBox::from_size(amap.size()), area.min, theme.foreground, kahlo::colour::BlendMode::SourceAlpha);
        }
    }
}
